<?php

try {
    require_once(dirname(__FILE__) . '/application.php');
    require_once(dirname(__FILE__) . '/setup.php');
    require_once(dirname(__FILE__) . '/lib/schema.php');
    require_once(dirname(__FILE__) . '/lib/fix.php');

    class Surfacer_FixBlogAlign extends Surfacer
    {
        function buildSurface(& $Tpl, $root)
        {
            $DB = DB::singleton(SetupCommon::dsn());
            $SQL = SQL::newSelect('blog');
            $SQL->setSelect('blog_id');
            $SQL->setGroup('blog_id');
            $SQL->setHaving(SQL::newOpr(SQL::newFunction('blog_id', 'count'), 2, '>='));
            $SQL->setLimit(1);

            if ( !!($bid = intval($DB->query($SQL->get(SetupCommon::dsn()), 'one'))) ) {
                $SQL = SQL::newSelect('blog');
                $SQL->addWhereOpr('blog_id', $bid);
                $i = 1;
                foreach ( $DB->query($SQL->get(SetupCommon::dsn()), 'all') as $row ) {
                    $Tpl->add(array('status:touch#', 'delete', $root) . $row['blog_status']);
                    $Tpl->add(array('indexing:touch#', 'delete', $root) . $row['blog_indexing']);
                    $Tpl->add(array('blog:loop', 'delete', $root), array(
                        'i' => $i++,
                        'bid' => $row['blog_id'],
                        'code' => $row['blog_code'],
                        'status' => $row['blog_status'],
                        'parent' => $row['blog_parent'],
                        'sort' => $row['blog_sort'],
                        'left' => $row['blog_left'],
                        'right' => $row['blog_right'],
                        'name' => $row['blog_name'],
                        'domain' => $row['blog_domain'],
                        'indexing' => $row['blog_indexing'],
                    ));
                }
            } else {
                $Tpl->add(array('align', $root));
            }
        }

        function validate()
        {

            if ( $this->Post->get('action') == 'delete' ) {
                if ( $this->Post->isNull('i') ) {
                    $this->Error['bid_input'] = true;
                }

            } elseif ( $this->Post->get('action') == 'align' ) {

            } else {

            }

            $this->detectStep();
        }

        function business()
        {
            $args = array();
            if ( $this->Post->get('action') == 'delete' ) {
                $args['selected'] = intval($this->Post->get('i'));
                $args['count'] = count($_POST['count']);

                $Fix = new Fix_BlogDelete($args);

                if ( !$Fix->execute($this->Post) ) $this->Error['execute'] = true;
            } elseif ( $this->Post->get('action') == 'align' ) {
                $Fix = new Fix_BlogAlign($args);

                if ( !$Fix->execute() ) $this->Error['execute'] = true;
            } else {

            }

            $this->detectStep();
        }
    }

    class Surfacer_FixCategoryAlign extends Surfacer
    {
        function buildSurface(& $Tpl, $root)
        {
            /*
            * list blog
            */
            $DB = DB::singleton(SetupCommon::dsn());
            $SQL = SQL::newSelect('blog');
            $all = $DB->query($SQL->get(SetupCommon::dsn()), 'all');

            foreach ( $all as $row ) {
                $vars = array(
                    'bid' => $row['blog_id'],
                    'name' => $row['blog_name'],
                    'code' => $row['blog_code'],
                    'parent' => $row['blog_parent'],
                );
                $Tpl->add(array('blog:loop', $root), $vars);
            }

        }

        function validate()
        {

            if ( $this->Post->isNull('bid') ) {
                $this->Error['bid_input'] = true;
            }

            $this->detectStep();
        }

        function business()
        {
            $args = array('bid' => $this->Post->get('bid'));

            $Fix = new Fix_CategoryAlign($args);

            if ( !$Fix->execute() ) $this->Error['execute'] = true;

            $this->detectStep();
        }
    }


    class Surfacer_FixCommentAlign extends Surfacer
    {
        function buildSurface(& $Tpl, $root)
        {
            /*
            * list blog
            */
            $DB = DB::singleton(SetupCommon::dsn());
            $SQL = SQL::newSelect('blog');
            $all = $DB->query($SQL->get(SetupCommon::dsn()), 'all');

            foreach ( $all as $row ) {
                $vars = array(
                    'bid' => $row['blog_id'],
                    'name' => $row['blog_name'],
                    'code' => $row['blog_code'],
                    'parent' => $row['blog_parent'],
                );
                $Tpl->add(array('blog:loop', $root), $vars);
            }

        }

        function validate()
        {

            if ( $this->Post->isNull('bid') ) {
                $this->Error['bid_input'] = true;
            }

            $this->detectStep();
        }

        function business()
        {
            $args = array('bid' => $this->Post->get('bid'));

            $Fix = new Fix_CommentAlign($args);

            if ( !$Fix->execute() ) $this->Error['execute'] = true;

            $this->detectStep();
        }
    }

    class Surfacer_FixDomain extends Surfacer
    {
        function buildSurface(& $Tpl, $root)
        {

        }

        function validate()
        {

            if ( $this->Post->isNull('domain') ) {
                $this->Error['domain_input'] = true;
            } elseif ( !preg_match(REGEX_VALID_DOMAIN, $this->Post->get('domain')) ) {
                $this->Error['domain_invalid'] = true;
            }

            if ( !SetupCommon::checkPermission(PATH_CONFIG, '666') ) $this->Error['config_permission'] = true;

            $this->detectStep();
        }

        function business()
        {
            $DB = DB::singleton(SetupCommon::dsn());

            $domain = $this->Post->get('domain');
            $all_blog = $this->Post->get('all_blog');

            $SQL = SQL::newUpdate('blog');
            $SQL->addUpdate('blog_domain', $domain);
            if ( $all_blog !== 'on' ) {
                $SQL->addWhereOpr('blog_id', 1);
            }
            $DB->query($SQL->get(SetupCommon::dsn()), 'exec');

            if ( !SetupCommon::rebuildConfig(array('DOMAIN' => $domain)) ) $this->Error['rewrite_config'] = true;

            if ( empty($this->Error) ) $this->Trans['domain'] = $this->Post->get('domain');

            $this->detectStep();
        }
    }

    class Surfacer_FixIndex extends Surfacer
    {
        function buildSurface(& $Tpl, $root)
        {

        }

        function validate()
        {
            if ( !SetupCommon::checkAlterPerm() ) {
                $this->Error['alter_privilege'] = true;
            }

            $this->detectStep();
        }

        function business()
        {
            $Schema = new Schema(SetupCommon::dsn(), DB_NAME, DB_PREFIX);
            $Schema->defSetYaml('schema');
            $defs = Schema::listUp($Schema->define);
            foreach ( $defs as $def ) {
                $Schema->clearIndex($def);
            }
            $Schema->makeIndex($Schema->defLoadYaml('index'));

            $this->detectStep();
        }
    }

    class Surfacer_FixSequence extends Surfacer
    {
        function buildSurface(& $Tpl, $root)
        {

        }

        function validate()
        {
            $this->detectStep();
        }

        function business()
        {
            $DB = DB::singleton(SetupCommon::dsn());
            $seq = SetupCommon::getSequence();

            foreach ( $seq as $fd => $val ) {
                if ( $fd == 'system_version' ) continue;
                $fd_ = $fd;
                $SQL = SQL::newSelect(str_replace('_id', '', $fd));
                if ( preg_match('/^shop_/', $fd) ) {
                    $fd = preg_replace('/^shop_/', '', $fd);
                }
                $SQL->addSelect($fd, $fd . '_max', null, 'max');
                $seq[$fd_] = $DB->query($SQL->get(SetupCommon::dsn()), 'one');
                if ( $seq[$fd_] == null ) $seq[$fd_] = 0;
            }

            $SQL = SQL::newUpdate('sequence');
            foreach ( $seq as $key => $val ) {
                $SQL->addUpdate('sequence_' . $key, $val);
            }
            $res = $DB->query($SQL->get(SetupCommon::dsn()), 'exec');
            if ( empty($res) ) $this->Error['update_failed'] = true;

            $this->detectStep();
        }
    }

    class Surfacer_FixSummary extends Surfacer
    {
        function buildSurface(& $Tpl, $root)
        {

        }

        function validate()
        {
            $this->detectStep();
        }

        function business()
        {
            @set_time_limit(0);

            $DB = DB::singleton(SetupCommon::dsn());
            $SQL = SQL::newSelect('blog');
            $SQL->addSelect('blog_id', 'id');
            $ids = $DB->query($SQL->get(SetupCommon::dsn()), 'all');


            foreach ( $ids as $blog ) {
                $SQL = SQL::newSelect('fulltext');
                $SQL->addSelect('fulltext_value');
                $SQL->addSelect('fulltext_eid');
                $SQL->addSelect('fulltext_cid');
                $SQL->addSelect('fulltext_uid');
                $SQL->addSelect('fulltext_bid');
                $SQL->addSelect('fulltext_blog_id');
                $SQL->addWhereOpr('fulltext_blog_id', $blog['id']);
                $all = $DB->query($SQL->get(SetupCommon::dsn()), 'all');

                foreach ( $all as $row ) {
                    $eid = $row['fulltext_eid'];
                    $cid = $row['fulltext_cid'];
                    $uid = $row['fulltext_uid'];
                    $bid = $row['fulltext_bid'];
                    $txt = SetupCommon::fixCRLF($row['fulltext_value']);

                    $SQL = SQL::newUpdate('fulltext');
                    $SQL->addUpdate('fulltext_value', $txt);
                    $SQL->addWhereOpr('fulltext_eid', $eid);
                    $SQL->addWhereOpr('fulltext_cid', $cid);
                    $SQL->addWhereOpr('fulltext_uid', $uid);
                    $SQL->addWhereOpr('fulltext_bid', $bid);
                    $SQL->addWhereOpr('fulltext_blog_id', $blog['id']);
                    $res = $DB->query($SQL->get(SetupCommon::dsn()), 'exec');
                }
            }
            $this->detectStep();
        }
    }

    class Surfacer_FixImage extends Surfacer
    {
        function buildSurface(& $Tpl, $root)
        {
            $step = $this->Post->get('step', 'apply');
            $DB = DB::singleton(SetupCommon::dsn());

            if ( $step == 'apply' ) {
                // list blog
                $SQL = SQL::newSelect('blog');
                $all = $DB->query($SQL->get(SetupCommon::dsn()), 'all');

                foreach ( $all as $row ) {
                    $vars = array(
                        'bid' => $row['blog_id'],
                        'name' => $row['blog_name'],
                        'code' => $row['blog_code'],
                        'parent' => $row['blog_parent'],
                    );
                    $Tpl->add(array('blog:loop', $root), $vars);
                }
            } else if ( $step == 'setting' ) {

                $bid = $this->Post->get('bid');
                $BlogConfig = loadBlogConfig($bid);
                $DefConfig = loadDefaultConfig();

                $SQL = SQL::newSelect('column');
                $SQL->addWhereOpr('column_type', 'image');
                $SQL->addWhereOpr('column_blog_id', $bid);
                $SQL->addSelect('column_size');
                $SQL->addGroup('column_size');
                $all = $DB->query($SQL->get(SetupCommon::dsn()), 'all');

                foreach ( $all as $row ) {
                    if ( empty($row['column_size']) ) continue;
                    $Tpl->add(array('normalSize:loop', $root), array(
                        'size' => $row['column_size'],
                    ));
                }

                $vars = array(
                    'largeSize' => $BlogConfig->get('image_size_large') ? $BlogConfig->get('image_size_large') : $DefConfig['image_size_large'],
                    'tinySize' => $BlogConfig->get('image_size_tiny') ? $BlogConfig->get('image_size_tiny') : $DefConfig['image_size_tiny'],
                    'squareSize' => $BlogConfig->get('image_size_square') ? $BlogConfig->get('image_size_square') : $DefConfig['image_size_square'],
                );
                $Tpl->add(array('size', $root), $vars);
            }
        }

        // バリデート
        function validate()
        {
            $step = $this->Post->get('step', 'apply');

            if ( $step == 'setting' ) {
                if ( $this->Post->isNull('bid') ) {
                    $this->Error['bid_input'] = true;
                    $this->Post->set('step', 'apply');
                }
            } else if ( $step == 'result' ) {
                $type = $this->Post->get('type');
                $resize = $this->Post->get('resize');

                if ( empty($type) ) {
                    $this->Error['type'] = true;
                }

                if ( empty($resize) ) {
                    $this->Error['resize'] = true;
                }

                if ( !empty($this->Error) ) {
                    $this->Post->set('step', 'setting');
                }
            }
        }

        // 実際の処理
        function business()
        {
            $LIB_DIR = '/' == substr(PHP_DIR, 0, 1) ? PHP_DIR : SCRIPT_DIR . PHP_DIR;
            require $LIB_DIR . 'ACMS/POST.php';

            $bid = $this->Post->get('bid');
            $type = $this->Post->get('type');
            $resize = $this->Post->get('resize');
            $normalSize = $this->Post->get('normalSize');
            $stdSide = $this->Post->get('imageSizeCriterion');
            $path = '';
            $Config = loadBlogConfig(1);
            $DefConfig = loadDefaultConfig();
            $imagic = $Config->get('image_magick', 'on');
            $quality = $Config->get('image_jpeg_quality') ? $Config->get('image_jpeg_quality') : $DefConfig['image_jpeg_quality'];

            $DB = DB::singleton(SetupCommon::dsn());

            $SQL = SQL::newSelect('column');
            $SQL->addSelect('column_id');
            $SQL->addSelect('column_field_2');
            $SQL->addWhereOpr('column_type', 'image');
            $SQL->addWhereOpr('column_blog_id', $bid);
            if ( $type == 'normal' ) {
                $SQL->addWhereOpr('column_size', $normalSize);
            }
            $all = $DB->query($SQL->get(SetupCommon::dsn()), 'all');

            foreach ( $all as $row ) {
                $path = $row['column_field_2'];
                $pfx = ('normal' == $type) ? '' : $type . '-';
                $target = '';

                // 各種サイズ
                $target = preg_replace('@(.*/)([^/]*)$@', '$1' . $pfx . '$2', $path);
                if ( !preg_match('@\.([^.]+)$@', $target, $match) ) continue;
                $ext = $match[1];

                $_file = SCRIPT_DIR . ARCHIVES_DIR . $target;

                // Large
                $_largePath = preg_replace('@(.*/)([^/]*)$@', '$1large-$2', $path);
                
                if (file_exists(SCRIPT_DIR . ARCHIVES_DIR . $_largePath)) {
                    if ($xy = getimagesize(SCRIPT_DIR . ARCHIVES_DIR . $_largePath)) {
                        $target = $_largePath;
                    }
                }

                $_width = null;
                $_height = null;
                $_size = $resize;
                $_angle = null;

                $editTarget = SCRIPT_DIR . ARCHIVES_DIR . $target;
                $_stdSide = $stdSide;

                // long side
                if (file_exists($editTarget)) {
                    if ($xy = getimagesize($editTarget)) {
                        if ($xy[0] >= $xy[1]) {
                            $stdSide = 'width';
                        } else {
                            $stdSide = 'height';
                        }
                    } else {
                        continue;
                    }
                } else {
                    continue;
                }
                
                // square
                if ( $type == 'square' ) {
                    $_width = $_size;
                    $_height = $_size;
                    // normal, tiny, large
                } else {
                    if ( $stdSide === 'w' || $stdSide === 'width' ) {
                        $_width = $_size;
                        $_size = null;
                    }
                    if ( $stdSide === 'h' || $stdSide === 'height' ) {
                        $_height = $_size;
                        $_size = null;
                    }
                }

                if ( class_exists('Imagick') && $imagic == 'on' ) {
                    ACMS_POST::editImageForImagick($editTarget, $_file, $_width, $_height, $_size, $_angle);
                } else if ( 'gif' == $ext ) {
                    imagegif(ACMS_POST::editImage(
                        imagecreatefromgif($editTarget), $_width, $_height, $_size, $_angle
                    ), $_file);
                } else if ( 'png' == $ext ) {
                    imagepng(ACMS_POST::editImage(
                        imagecreatefrompng($editTarget), $_width, $_height, $_size, $_angle
                    ), $_file);
                } else if ( 'bmp' == $ext ) {
                    imagewbmp(ACMS_POST::editImage(
                        imagecreatefromwbmp($editTarget), $_width, $_height, $_size, $_angle
                    ), $_file);
                } else if ( 'xbm' == $ext ) {
                    imagexbm(ACMS_POST::editImage(
                        imagecreatefromxbm($editTarget), $_width, $_height, $_size, $_angle
                    ), $_file);
                } else {
                    imagejpeg(ACMS_POST::editImage(
                        imagecreatefromjpeg($editTarget), $_width, $_height, $_size, $_angle
                    ), $_file, intval($quality));
                }

                // DB Update
                if ( $type == 'normal' ) {
                    $SQL = SQL::newUpdate('column');
                    $SQL->addUpdate('column_size', $_stdSide . $resize);
                    $SQL->addWhereOpr('column_type', 'image');
                    $SQL->addWhereOpr('column_blog_id', $bid);
                    $SQL->addWhereOpr('column_id', $row['column_id']);
                    $DB->query($SQL->get(SetupCommon::dsn()), 'exec');
                }
            }
        }
    }

    class Surfacer_FixAccess extends Surfacer
    {
        function buildSurface(& $Tpl, $root)
        {
            /*
            * list blog
            */
            $DB = DB::singleton(SetupCommon::dsn());
            $SQL = SQL::newSelect('blog');
            $all = $DB->query($SQL->get(SetupCommon::dsn()), 'all');

            foreach ( $all as $row ) {
                $vars = array(
                    'bid' => $row['blog_id'],
                    'name' => $row['blog_name'],
                    'code' => $row['blog_code'],
                    'parent' => $row['blog_parent'],
                );
                $Tpl->add(array('blog:loop', $root), $vars);
            }
        }

        function validate()
        {
            if ( $this->Post->isNull('bid') ) {
                $this->Error['bid_input'] = true;
            }

            $this->detectStep();
        }

        function business()
        {
            $bid = $this->Post->get('bid');
            $DB = DB::singleton(SetupCommon::dsn());
            $delete_target = array(
                'login_white_hosts',
                'login_black_hosts',
                'basic_auth_user',
                'basic_auth_pass',
                'basic_auth_skip_hosts',
            );

            $SQL = SQL::newDelete('config');
            $SQL->addWhereOpr('config_blog_id', $bid);
            $SQL->addWhereIn('config_key', $delete_target);
            $DB->query($SQL->get(SetupCommon::dsn()), 'exec');

            $this->detectStep();
        }
    }

    class Surfacer_FixFieldSearch extends Surfacer
    {
        function buildSurface(& $Tpl, $root)
        {

        }

        function validate()
        {
            $this->detectStep();
        }

        function business()
        {
            $DB = DB::singleton(SetupCommon::dsn());

            $SQL = SQL::newUpdate('field');
            $SQL->addUpdate('field_search', 'on');
            $SQL->addWhereOpr('field_key', '%@%', 'NOT LIKE');
            $DB->query($SQL->get(SetupCommon::dsn()), 'exec');

            $this->detectStep();
        }
    }

    class Surfacer_FixFulltext extends Surfacer
    {
        function buildSurface(& $Tpl, $root)
        {

        }

        function validate()
        {
            if ( 0
                || $this->Post->isNull('type')
                || !in_array($this->Post->get('type'), array('blog', 'category', 'user', 'entry'))
            ) {
                $this->Error['type_input'] = true;
            }
            $this->detectStep();
        }

        function business()
        {
            $type = $this->Post->get('type');
            $Fix = new Fix_Fulltext(array('type' => $type));

            if ( !$Fix->execute() ) $this->Error['execute'] = true;

            $this->detectStep();
        }
    }

    /**
     * Detect Switching Call Class
     */

    switch ( htmlentities($_GET['c'], ENT_QUOTES, 'UTF-8') ) {
        case 'blog-align' :
            $tplPath = dirname(__FILE__) . "/tpl/fix/blog-align.html";
            new Surfacer_FixBlogAlign($tplPath);
            break;
        case 'category-align' :
            $tplPath = dirname(__FILE__) . "/tpl/fix/category-align.html";
            new Surfacer_FixCategoryAlign($tplPath);
            break;
        case 'comment-align' :
            $tplPath = dirname(__FILE__) . "/tpl/fix/comment-align.html";
            new Surfacer_FixCommentAlign($tplPath);
            break;
        case 'domain' :
            $tplPath = dirname(__FILE__) . "/tpl/fix/domain.html";
            new Surfacer_Fixdomain($tplPath);
            break;
        case 'index' :
            $tplPath = dirname(__FILE__) . "/tpl/fix/index.html";
            new Surfacer_FixIndex($tplPath);
            break;
        case 'field-search' :
            $tplPath = dirname(__FILE__) . "/tpl/fix/field-search.html";
            new Surfacer_FixFieldSearch($tplPath);
            break;
        case 'fulltext' :
            $tplPath = dirname(__FILE__) . "/tpl/fix/fulltext.html";
            new Surfacer_FixFulltext($tplPath);
            break;
        case 'sequence' :
            $tplPath = dirname(__FILE__) . "/tpl/fix/sequence.html";
            new Surfacer_FixSequence($tplPath);
            break;
        case 'summary' :
            $tplPath = dirname(__FILE__) . "/tpl/fix/summary.html";
            new Surfacer_FixSummary($tplPath);
            break;
        case 'access' :
            $tplPath = dirname(__FILE__) . "/tpl/fix/access.html";
            new Surfacer_FixAccess($tplPath);
            break;
        case 'image' :
            $tplPath = dirname(__FILE__) . "/tpl/fix/image.html";
            new Surfacer_FixImage($tplPath);
            break;
        default :
            header('Content-type: text/html; charset=UTF-8');
            die('<p>不正な命令です</p>');
            break;
    }

    App::checkException();
} catch ( Exception $e ) {
    App::showError($e);
}